# frozen_string_literal: true

module Izokatu
  # Izokatu selector of options for selected action class
  class ActionCallOptionsSelector
    extend Callable

    include Contracts

    # @return [Class] selected action class
    attr_reader :action_class
    # @return [Symbol] action to execute
    attr_reader :action
    # @return [Symbol] library used for encryption/decryption
    attr_reader :via
    # @return [String] OpenSSL private key cipher
    # @note also used for OpenSSL public key ec key generation
    attr_reader :cipher
    # @return [String] string of clear data for encryption
    attr_reader :clear_data_string
    # @return [String] string of encrypted data for decryption
    attr_reader :encrypted_data_string
    # @return [RBNACL_KEY_CLASSES || OPENSSL_KEY_CLASSES] public key for public key encryption/decryption
    attr_reader :public_key
    # @return [RBNACL_KEY_CLASSES || OPENSSL_KEY_CLASSES] private key for public key encryption/decryption
    attr_reader :private_key
    # @return [String] encryption key for private key encryption/decryption
    attr_reader :key
    # @return [String] initialization vector for one-time use
    attr_reader :nonce
    # @return [String] authenticated data
    attr_reader :auth_data
    # @return [String] authentication tag
    attr_reader :auth_tag
    # @return [Hash] options for OpenSSL public key EC encryption/decryption
    attr_reader :ecies_options
    # @return [Integer] bit number for OpenSSL public key RSA encryption/decryption
    attr_reader :bit_number

    # RbNaCl public key classes, used for contracts
    RBNACL_KEY_CLASSES = [
      RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey,
      RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey
    ].freeze

    # OpenSSL public key classes, used for contracts
    OPENSSL_KEY_CLASSES = [
      OpenSSL::PKey::RSA,
      OpenSSL::PKey::EC
    ].freeze

    Contract Contracts::HashOf[Symbol, Or[Class, Contracts::HashOf[Symbol, Any]]] => Any
    # Initializing options for action class
    #
    # @param action_class (#action_class)
    # @param options (#options)
    #
    # @since 0.1.0
    def initialize(action_class:, options:)
      @action_class = action_class.to_s
      @action = options[:action]
      @via = options[:via]
      @cipher = options[:cipher]
      @clear_data_string = options[:clear_data_string]
      @encrypted_data_string = options[:encrypted_data_string]
      @public_key = options[:public_key]
      @private_key = options[:private_key]
      @key = options[:key]
      @nonce = options[:nonce]
      @auth_data = options[:auth_data]
      @auth_tag = options[:auth_tag]
      @ecies_options = options[:ecies_options]
      @bit_number = options[:bit_number]
    end

    Contract None => Contracts::HashOf[Symbol, Any]
    # Selecting options for keys generation class or for encryption/decryption class
    #
    # @return [Hash] options for action class call
    #
    # @since 0.1.0
    def perform
      action == :keys_generation ? select_keys_generation_action_options : select_default_action_options
    end

    private

    Contract None => Or[{}, Contracts::HashOf[Symbol, Or[String, Pos]]]
    # Selecting options for keys generation class
    #
    # @return [Hash] options for keys generation class
    #
    # @since 0.1.0
    def select_keys_generation_action_options
      case action_class
      when 'Izokatu::Rbnacl::PublicKey::KeysGenerator'
        {}
      when 'Izokatu::Openssl::PublicKey::RSA::KeysGenerator'
        { bit_number: bit_number }
      when 'Izokatu::Openssl::PublicKey::EC::KeysGenerator'
        { cipher: cipher }
      end
    end

    Contract None => Contracts::HashOf[Symbol, Or[*RBNACL_KEY_CLASSES, *OPENSSL_KEY_CLASSES, String, nil]]
    # Selecting options for encryption/decryption class
    #
    # @return [Hash] options for encryption/decryption class
    #
    # @since 0.1.0
    def select_default_action_options
      via == :rbnacl ? select_rbnacl_action_options : select_openssl_action_options
    end

    Contract None => Contracts::HashOf[Symbol, Or[*RBNACL_KEY_CLASSES, String, nil]]
    # Selecting options for Rbnacl encryption/decryption class
    #
    # @return [Hash] options for Rbnacl encryption/decryption class
    #
    # @since 0.1.0
    def select_rbnacl_action_options
      case action_class
      when 'Izokatu::Rbnacl::PrivateKey::Encrypter'
        { auth_data: auth_data, clear_data: clear_data_string }
      when 'Izokatu::Rbnacl::PrivateKey::Decrypter'
        {
          encrypted_data: encrypted_data_string,
          nonce: nonce,
          key: key,
          auth_data: auth_data
        }
      when 'Izokatu::Rbnacl::PublicKey::Encrypter'
        {
          public_key: public_key,
          private_key: private_key,
          clear_data: clear_data_string
        }
      when 'Izokatu::Rbnacl::PublicKey::Decrypter'
        {
          encrypted_data: encrypted_data_string,
          nonce: nonce,
          public_key: public_key,
          private_key: private_key
        }
      end
    end

    Contract None => Contracts::HashOf[Symbol, Or[*OPENSSL_KEY_CLASSES, String, nil]]
    # Selecting options for Openssl encryption/decryption class
    #
    # @return [Hash] options for Openssl encryption/decryption class
    #
    # @since 0.1.0
    def select_openssl_action_options
      case action_class
      when 'Izokatu::Openssl::PrivateKey::Default::Encrypter'
        { cipher: cipher, clear_data: clear_data_string }
      when 'Izokatu::Openssl::PrivateKey::Default::Decrypter'
        {
          cipher: cipher,
          encrypted_data: encrypted_data_string,
          key: key,
          nonce: nonce
        }
      when 'Izokatu::Openssl::PrivateKey::Auth::Encrypter'
        {
          cipher: cipher,
          clear_data: clear_data_string,
          auth_data: auth_data
        }
      when 'Izokatu::Openssl::PrivateKey::Auth::Decrypter'
        {
          cipher: cipher,
          encrypted_data: encrypted_data_string,
          key: key,
          nonce: nonce,
          auth_data: auth_data,
          auth_tag: auth_tag
        }
      when 'Izokatu::Openssl::PrivateKey::Auth::CCM::Encrypter'
        {
          cipher: cipher,
          clear_data: clear_data_string,
          auth_data: auth_data
        }
      when 'Izokatu::Openssl::PrivateKey::Auth::CCM::Decrypter'
        {
          cipher: cipher,
          encrypted_data: encrypted_data_string,
          key: key,
          nonce: nonce,
          auth_data: auth_data,
          auth_tag: auth_tag
        }
      when 'Izokatu::Openssl::PublicKey::RSA::Encrypter'
        { clear_data: clear_data_string, public_key: public_key }
      when 'Izokatu::Openssl::PublicKey::RSA::Decrypter'
        { private_key: private_key, encrypted_data: encrypted_data_string }
      when 'Izokatu::Openssl::PublicKey::EC::Encrypter'
        {
          clear_data: clear_data_string,
          public_key: public_key,
          ecies_options: ecies_options
        }
      when 'Izokatu::Openssl::PublicKey::EC::Decrypter'
        {
          private_key: private_key,
          encrypted_data: encrypted_data_string,
          ecies_options: ecies_options
        }
      end
    end
  end
end
