# frozen_string_literal: true

module Izokatu
  # Izokatu helper methods
  module Helpers
    include Contracts

    # Default Symbol keys for keys
    KEY_SYMBOL = %i[private_key public_key].freeze
    # Rbnacl classes for keys
    RBNACL_KEY_CLASSES = [
      RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey,
      RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey
    ].freeze

    # Verify cipher is included in OpenSSL public key EC ciphers
    EC_CIPHER = ->(c) { Izokatu::Openssl::PBKEY_EC_CIPHERS.include?(c) }

    Contract EC_CIPHER => Contracts::HashOf[Or[*KEY_SYMBOL], String]
    # Generating EC keypair
    #
    # @param cipher [String] cipher for keys encryption
    #
    # @return [Hash] keypair of EC public and private key
    #
    # @since 0.1.0
    def generate_ec_keypair(cipher = 'secp521r1')
      Izokatu::Openssl::PublicKey::EC::KeysGenerator.call(cipher: cipher)
    end

    Contract Pos => Contracts::HashOf[Or[*KEY_SYMBOL], String]
    # Generating RSA keypair
    #
    # @param bit_number [Integer] number of key bits
    #
    # @return [Hash] keypair of RSA public and private key
    #
    # @since 0.1.0
    def generate_rsa_keypair(bit_number = 4096)
      Izokatu::Openssl::PublicKey::RSA::KeysGenerator.call(bit_number: bit_number)
    end

    Contract None => Contracts::HashOf[Or[*KEY_SYMBOL], Or[*RBNACL_KEY_CLASSES]]
    # Generating RbNaCl keypair
    #
    # @return [Hash] keypair of RbNaCl public and private key
    #
    # @since 0.1.0
    def generate_rbnacl_keypair
      Izokatu::Rbnacl::PublicKey::KeysGenerator.call
    end

    Contract None => Contracts::HashOf[Symbol, Or[String, Symbol, Bool]]
    # Performing encryption and merging result in options
    #
    # @since 0.1.0
    def import_encrypted_in_options!
      encrypted_data, decrypter_params = encrypt(options)
      options.merge!(encrypted_data)
      options.merge!(decrypter_params)
    end

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, Maybe[String]], String, Bool]] =>
    Maybe[Contracts::HashOf[Symbol, Maybe[String]]]
    # Exporting encrypted data and decrypter params
    #
    # @param encrypted_data [Hash] Hash with encrypted data for export
    # @param decrypter_params [Hash] Hash with decrypter params for export
    # @param encode [TrueClass || FalseClass] Enable/disable encoding of exported data
    #
    # @return [Hash] merged encrypted data and decrypter params
    #
    # @since 0.1.0
    def export_encrypted!(encrypted_data:, decrypter_params:, encode:)
      encrypted = export_data(
        data: encrypted_data,
        filename: options[:encrypted_data_filename],
        encode: encode
      )
      params = export_data(
        data: decrypter_params,
        filename: options[:decrypter_params_filename],
        encode: encode
      )

      encrypted && params ? encrypted.merge(params) : nil
    end

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, String], String, Bool]] =>
    Maybe[Contracts::HashOf[Symbol, String]]
    # Exporting decrypted data
    #
    # @param decrypted_data [Hash] Hash with encrypted data for export
    # @param encode [TrueClass || FalseClass] Enable/disable encoding of exported data
    #
    # @return [Hash] decrypted data
    #
    # @note Returning value even if not using :function exporter
    #
    # @since 0.1.0
    def export_decrypted!(decrypted_data:, encode:)
      export_data(
        data: decrypted_data,
        filename: options[:decrypted_data_filename],
        encode: encode
      )
    end

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, Maybe[String]], Maybe[String], Bool]] =>
    Maybe[Contracts::HashOf[Symbol, Maybe[String]]]
    # Exporting data
    #
    # @param data [Hash] Hash with data for export
    # @param filename [String] Name of file for export
    # @param encode [TrueClass || FalseClass] Enable/disable encoding of exported data
    #
    # @return [Hash] exported data
    #
    # @note Returning value even if not using :function exporter
    #
    # @since 0.1.0
    def export_data(data:, filename:, encode:)
      other_options = { data: data, encode: encode }
      file_options = {
        data: data,
        filename: filename,
        encode: encode
      }
      exporter = options[:exporter]
      exporter_options = exporter == Izokatu::FileExporter ? file_options : other_options
      exporter.call(**exporter_options)
    end

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, Any], Bool]] =>
    Contracts::HashOf[Symbol, Any]
    # Importing encrypted data and decrypter params
    #
    # @param options [Hash] Hash with options
    # @param decode [TrueClass || FalseClass] Enable/disable decoding of imported data
    #
    # @return [Hash] Hash with updated options
    #
    # @since 0.1.0
    def import_encrypted!(options:, decode:)
      encrypted_data = options.select { |k, _v| k == :encrypted_data_string }
      decrypter_params_default_keys = %i[nonce key auth_data auth_tag]
      decrypter_params = options.select { |k, _v| decrypter_params_default_keys.include?(k) }
      encrypted = import_data(
        data: encrypted_data,
        filename: options[:encrypted_data_filename],
        delete_imported: options[:delete_imported],
        decode: decode
      )
      params = options.merge! import_data(
        data: decrypter_params,
        filename: options[:decrypter_params_filename],
        delete_imported: options[:delete_imported],
        decode: decode
      )
      options[:encrypted_data_string] = encrypted.values[0]
      options.merge!(params)
    end

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, String], Any]] =>
    Contracts::HashOf[Symbol, Maybe[String]]
    # Importing data
    #
    # @param data [Hash] Hash with data to import from
    # @param filename [String] Name of file to import from
    # @param delete_imported [TrueClass, FalseClass] Enable/disable deleting file after import
    # @param decode [TrueClass || FalseClass] Enable/disable decoding of imported data
    #
    # @return [Hash] imported data
    #
    # @since 0.1.0
    def import_data(data:, filename:, delete_imported:, decode:)
      function_options = { data: data, decode: decode }
      file_options = {
        filename: filename,
        delete_imported: delete_imported,
        decode: decode
      }
      importer = options[:importer]
      importer_options = importer == Izokatu::FunctionImporter ? function_options : file_options
      importer.call(**importer_options)
    end

    Contract Contracts::HashOf[Symbol, Maybe[String]] =>
    Contracts::HashOf[Symbol, Maybe[String]]
    # Encoding data
    #
    # @param data [Hash] Hash with data for encoding
    #
    # @return [Hash] encoded data
    #
    # @since 0.1.0
    def encode_data(data)
      data.transform_values { |v| Base64.strict_encode64(v) if v }
    end

    Contract Contracts::HashOf[Symbol, Maybe[String]] =>
    Contracts::HashOf[Symbol, Maybe[String]]
    # Decoding data
    #
    # @param data [Hash] Hash with data for decoding
    #
    # @return [Hash] decoded data
    #
    # @since 0.1.0
    def decode_data(data)
      data.transform_values { |v| Base64.strict_decode64(v) if v }
    end
  end
end
