# frozen_string_literal: true

module Izokatu
  # Abstract class for keys generators
  #
  # @abstract Subclasses are containing implementation of {#generate_private_key} and {#generate_public_key}
  class KeysGenerator
    extend Callable

    include Contracts

    # Default key symbols for RbNaCl and OpenSSL keys
    KEYS_SYMBOLS = %i[private_key public_key].freeze
    # Classes of RbNaCl and OpenSSL keys
    KEY_CLASSES = [
      RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey,
      RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey,
      OpenSSL::PKey::RSA,
      OpenSSL::PKey::EC
    ].freeze

    Contract None => Contracts::HashOf[Or[*KEYS_SYMBOLS], Or[*KEY_CLASSES]]
    # Performing generation of private and public keys
    #
    # @return [Hash] public and private keys
    #
    # @since 0.1.0
    def perform
      generate_private_key
        .then { |private_key| { private_key: private_key, public_key: generate_public_key(private_key) } }
    end

    private

    # Performing generation of private key
    #
    # @raise RuntimeError
    #
    # @since 0.1.0
    def generate_private_key
      raise 'Not implemented!'
    end

    # Performing generation of public key
    #
    # @raise RuntimeError
    #
    # @since 0.1.0
    def generate_public_key(_private_key)
      raise 'Not implemented!'
    end
  end
end
