# frozen_string_literal: true

module Izokatu
  module Openssl
    module PrivateKey
      module Auth
        # Namespace for OpenSSL private key authenticated CCM classes
        module CCM
          # OpenSSL private key decrypter for authenticated ciphers in CCM mode
          class Decrypter < Izokatu::Openssl::PrivateKey::Auth::Decrypter
            # Default iv (nonce) length for ciphers in CCM mode
            DEFAULT_CCM_IV_LENGTH = 7
            # Default authentication tag length for ciphers in CCM mode
            DEFAULT_CCM_AUTH_TAG_LENGTH = 8

            private

            Contract None => Any
            # Initializing decrypter params
            #
            # @since 0.1.0
            def initialize_decrypter_params!
              initialize_auth_ccm_decrypter_params!
              super
            end

            Contract None => Any
            # Set length for iv and auth_tag before their initialization
            #
            # @since 0.1.0
            def initialize_auth_ccm_decrypter_params!
              decrypter.iv_len = DEFAULT_CCM_IV_LENGTH
              decrypter.auth_tag_len = DEFAULT_CCM_AUTH_TAG_LENGTH
            end

            Contract None => Any
            # Initializing decrypter auth params
            #
            # @since 0.1.0
            def initialize_auth_decrypter_params!
              initialize_static_message_length!
              super
            end

            Contract None => Any
            # Initializing message length for cipher in CCM mode
            #
            # @since 0.1.0
            def initialize_static_message_length!
              decrypter.ccm_data_len = encrypted_data.length unless cipher.include?('ARIA')
            end

            # Raising exception if auth tag is truncated
            # @note Redefined to use local auth tag length
            #
            # @raise RuntimeError
            #
            # @since 0.1.0
            def verify_tag_size!
              raise 'tag is truncated!' unless auth_tag.bytesize == DEFAULT_CCM_AUTH_TAG_LENGTH
            end
          end
        end
      end
    end
  end
end
