# frozen_string_literal: true

module Izokatu
  module Openssl
    module PrivateKey
      # Namespace for OpenSSL private key authenticated classes
      module Auth
        # OpenSSL private key decrypter for authenticated ciphers
        class Decrypter < Izokatu::Openssl::PrivateKey::Default::Decrypter
          # @return [String] authenticated data
          attr_reader :auth_data
          # @return [String] authentication tag
          attr_reader :auth_tag

          # Default Openssl::PrivateKey::Auth::Decrypter option
          DEFAULT_AUTH_TAG_LENGTH = 16

          Contract Contracts::HashOf[Symbol, Maybe[String]] => Any
          # Initializing options for OpenSSL EC decryption
          #
          # @param encrypted_data (#encrypted_data)
          # @param cipher (#cipher)
          # @param key (#key)
          # @param nonce (#nonce)
          # @param auth_data (#auth_data)
          # @param auth_tag (#auth_tag)
          #
          # @since 0.1.0
          def initialize(auth_data:, auth_tag:, cipher:, encrypted_data:, key:, nonce:)
            @auth_data = auth_data
            @auth_tag = auth_tag
            super(cipher: cipher, encrypted_data: encrypted_data, key: key, nonce: nonce)
          end

          Contract None => Contracts::HashOf[Symbol, String]
          # Initializing decrypter
          #
          # @return [OpenSSL::Cipher] decrypter instance
          #
          # @since 0.1.0
          def perform
            verify_tag_size!
            super
          end

          private

          Contract None => Any
          # Initializing decrypter params
          #
          # @since 0.1.0
          def initialize_decrypter_params!
            super
            initialize_auth_decrypter_params!
          end

          Contract None => Any
          # Initializing decrypter auth params
          #
          # @since 0.1.0
          def initialize_auth_decrypter_params!
            # unless added just for ARIA-***-CCM ciphers
            decrypter.auth_data = auth_data unless cipher.include?('ARIA')
            decrypter.auth_tag = auth_tag
          end

          # Raising exception if auth tag is truncated
          #
          # @raise RuntimeError
          #
          # @since 0.1.0
          def verify_tag_size!
            raise 'tag is truncated!' unless auth_tag.bytesize == DEFAULT_AUTH_TAG_LENGTH
          end
        end
      end
    end
  end
end
