# frozen_string_literal: true

module Izokatu
  module Openssl
    module PrivateKey
      module Auth
        # OpenSSL private key encrypter for authenticated ciphers
        class Encrypter < Izokatu::Openssl::PrivateKey::Default::Encrypter
          # @return [String] authenticated data
          attr_reader :auth_data

          # Default Openssl::PrivateKey::Auth::Encrypter option
          DEFAULT_AUTH_OPTIONS = {
            auth_data: ''
          }.freeze

          Contract Contracts::HashOf[Symbol, Maybe[String]] => Any
          # Initializing options for OpenSSL EC encryption
          #
          # @param cipher (#cipher)
          # @param clear_data (#clear_data)
          # @param auth_data (#auth_data)
          #
          # @since 0.1.0
          def initialize(cipher:, clear_data:, auth_data:)
            super(cipher: cipher, clear_data: clear_data)
            initialize_auth_encrypter_params!(auth_data)
          end

          private

          Contract Maybe[String] => Maybe[String]
          # Initializing encrypter auth param
          #
          # @return [String] authenticated data
          #
          # @since 0.1.0
          def initialize_auth_encrypter_params!(auth_data)
            @auth_data = auth_data || DEFAULT_AUTH_OPTIONS[:auth_data]
            encrypter.auth_data = @auth_data
          end

          Contract None => Contracts::ArrayOf[Contracts::HashOf[Symbol, String]]
          # Encrypting data
          #
          # @return [Array] encrypted data with decrypter params
          #
          # @since 0.1.0
          def encrypt_data!
            encrypted_data, decrypter_params = super
            [encrypted_data, decrypter_params]
          end

          Contract None => Contracts::HashOf[Symbol, String]
          # Returning decrypter params
          #
          # @return [Hash] decrypter params
          #
          # @since 0.1.1
          def decrypter_params
            super.merge(auth_data: auth_data, **compute_auth_tag!)
          end

          Contract None => Contracts::HashOf[Symbol, String]
          # Computing authentication tag
          #
          # @return [Hash] authentication tag
          #
          # @since 0.1.0
          def compute_auth_tag!
            { auth_tag: encrypter.auth_tag }
          end
        end
      end
    end
  end
end
