# frozen_string_literal: true

module Izokatu
  module Openssl
    # Namespace for Openssl private key classes
    module PrivateKey
      # Namespace for OpenSSL private key non-authenticated classes
      module Default
        # OpenSSL private key decrypter for non-authenticated ciphers
        class Decrypter < Izokatu::Decrypter
          # @return [String] OpenSSL private key cipher
          attr_reader :cipher
          # @return [String] key for private key encryption/decryption
          attr_reader :key
          # @return [String] initialization vector for one-time use
          attr_reader :nonce

          # Default Openssl::PrivateKey::Default::Decrypter option
          DEFAULT_OPTIONS = {
            cipher: 'AES256'
          }.freeze

          Contract Contracts::HashOf[Symbol, String] => Any
          # Initialize options for OpenSSL EC decryption
          #
          # @param encrypted_data (#encrypted_data)
          # @param cipher (#cipher)
          # @param key (#key)
          # @param nonce (#nonce)
          #
          # @since 0.1.0
          def initialize(encrypted_data:, cipher:, key:, nonce:)
            super(encrypted_data: encrypted_data)
            @cipher = cipher || DEFAULT_OPTIONS[:cipher]
            @key = key
            @nonce = nonce
            create_decrypter!
            initialize_decrypter_params!
          end

          private

          Contract None => OpenSSL::Cipher
          # Initializing decrypter
          #
          # @return [OpenSSL::Cipher] decrypter instance
          #
          # @since 0.1.0
          def create_decrypter!
            @decrypter = OpenSSL::Cipher.new(cipher).decrypt
          end

          Contract None => Any
          # Initializing decrypter params
          #
          # @since 0.1.0
          def initialize_decrypter_params!
            decrypter.key = key
            decrypter.iv = nonce
          end

          Contract None => Contracts::HashOf[Symbol, String]
          # Decrypting data
          #
          # @return [Hash] decrypted data
          #
          # @since 0.1.0
          def decrypt_data!
            { decrypted_data_string: decrypter.update(encrypted_data) + decrypter.final }
          end
        end
      end
    end
  end
end
