# frozen_string_literal: true

module Izokatu
  module Openssl
    module PrivateKey
      module Default
        # OpenSSL private key encrypter for non-authenticated ciphers
        class Encrypter < Izokatu::Encrypter
          # @return [String] OpenSSL private key cipher
          attr_reader :cipher
          # @return [String] key for private key encryption/decryption
          attr_reader :key
          # @return [String] initialization vector for one-time use
          attr_reader :nonce

          # Default Openssl::PrivateKey::Default::Encrypter option
          DEFAULT_OPTIONS = {
            cipher: 'AES256'
          }.freeze

          Contract Contracts::HashOf[Symbol, String] => Any
          # Initializing options for OpenSSL EC encryption
          #
          # @param clear_data (#clear_data)
          # @param cipher (#cipher)
          #
          # @since 0.1.0
          def initialize(clear_data:, cipher:)
            super(clear_data: clear_data)
            @cipher = cipher || DEFAULT_OPTIONS[:cipher]
            create_encrypter!
            @key = encrypter.random_key
            @nonce = encrypter.random_iv
            initialize_encrypter_params!
          end

          private

          Contract None => OpenSSL::Cipher
          # Initializing encrypter
          #
          # @return [OpenSSL::Cipher] encrypter instance
          #
          # @since 0.1.0
          def create_encrypter!
            @encrypter = OpenSSL::Cipher.new(cipher).encrypt
          end

          Contract None => Any
          # Initializing encrypter params
          #
          # @since 0.1.0
          def initialize_encrypter_params!
            # OpenSSL::Cipher instances has only key=, iv= and auth_data= methods
            encrypter.key = key
            encrypter.iv = nonce
          end

          Contract None => Contracts::ArrayOf[Contracts::HashOf[Symbol, String]]
          # Encrypting data
          #
          # @return [Array] encrypted data with decrypter params
          #
          # @since 0.1.0
          def encrypt_data!
            [
              { encrypted_data_string: encrypter.update(clear_data) + encrypter.final },
              decrypter_params
            ]
          end

          Contract None => Contracts::HashOf[Symbol, String]
          # Returning decrypter params
          #
          # @return [Hash] decrypter params
          #
          # @since 0.1.1
          def decrypter_params
            { nonce: nonce, key: key }
          end
        end
      end
    end
  end
end
