# frozen_string_literal: true

module Izokatu
  module Openssl
    module PublicKey
      module EC
        # OpenSSL public key EC decrypter
        class Decrypter < Izokatu::Decrypter
          # @return [String] private key string for decryption
          attr_reader :private_key
          # @return [String] ECIES cipher name
          attr_reader :ecies_cipher
          # @return [String] ECIES digest name
          attr_reader :ecies_digest
          # @return [Symbol] ECIES MAC length
          attr_reader :ecies_mac_length
          # @return [String] ECIES KDF digest name
          attr_reader :ecies_kdf_digest
          # @return [String] ECIES MAC digest name
          attr_reader :ecies_mac_digest

          # Default options for ECIES
          DEFAULT_ECIES_OPTIONS = {
            ecies_cipher: 'AES-256-CTR',
            ecies_digest: 'SHA512',
            ecies_mac_length: :full,
            ecies_kdf_digest: 'SHA512',
            ecies_mac_digest: 'SHA512'
          }.freeze

          Contract Contracts::HashOf[Symbol, Maybe[String]] => Any
          def initialize(encrypted_data:, private_key:, ecies_options:)
            # Initialize options for OpenSSL EC decryption
            #
            # @param encrypted_data (#encrypted_data)
            # @param private_key (#private_key)
            # @param ecies_options Hash with ECIES options
            #
            # @since 0.1.0
            super(encrypted_data: encrypted_data)
            initialize_private_key!(private_key)
            initialize_ecies_options!(ecies_options || DEFAULT_ECIES_OPTIONS)
            initialize_decrypter!
          end

          Contract String => OpenSSL::PKey::EC
          # Initialize EC private key from private key string
          #
          # @param private_key (#private_key)
          #
          # @return [OpenSSL:PKey::EC] OpenSSL private key instance
          #
          # @since 0.1.0
          def initialize_private_key!(private_key)
            raise 'ERROR: No private key!' unless private_key

            @private_key = OpenSSL::PKey.read(private_key)
          end

          Contract Contracts::HashOf[Symbol, Or[String, Symbol]] => Any
          # Initialize ECIES options
          #
          # @param ecies_options Hash with ECIES options
          #
          # @since 0.1.0
          def initialize_ecies_options!(ecies_options)
            @ecies_cipher = ecies_options[:ecies_cipher]
            @ecies_digest = ecies_options[:ecies_digest]
            @ecies_mac_length = ecies_options[:ecies_mac_length]
            @ecies_kdf_digest = ecies_options[:ecies_kdf_digest]
            @ecies_mac_digest = ecies_options[:ecies_mac_digest]
          end

          Contract None => ECIES::Crypt
          # Initialize decrypter
          #
          # @return [ECIES::Crypt] decrypter instance
          #
          # @since 0.1.0
          def initialize_decrypter!
            @decrypter = ECIES::Crypt.new(
              cipher: ecies_cipher,
              digest: ecies_digest,
              mac_length: ecies_mac_length,
              kdf_digest: ecies_kdf_digest,
              mac_digest: ecies_mac_digest
            )
          end

          private

          Contract None => Contracts::HashOf[Symbol, String]
          # Decrypting data
          #
          # @return [Hash] decrypted data
          #
          # @since 0.1.0
          def decrypt_data!
            { decrypted_data_string: decrypter.decrypt(private_key, encrypted_data) }
          end
        end
      end
    end
  end
end
