# frozen_string_literal: true

module Izokatu
  module Openssl
    module PublicKey
      module EC
        # OpenSSL public key EC encrypter
        class Encrypter < Izokatu::Encrypter
          # @return [String] public key string for decryption
          attr_reader :public_key
          # @return [String] ECIES cipher name
          attr_reader :ecies_cipher
          # @return [String] ECIES digest name
          attr_reader :ecies_digest
          # @return [Symbol] ECIES MAC length
          attr_reader :ecies_mac_length
          # @return [String] ECIES KDF digest name
          attr_reader :ecies_kdf_digest
          # @return [String] ECIES MAC digest name
          attr_reader :ecies_mac_digest

          # Default options for ECIES
          DEFAULT_ECIES_OPTIONS = {
            ecies_cipher: 'AES-256-CTR',
            ecies_digest: 'SHA512',
            ecies_mac_length: :full,
            ecies_kdf_digest: 'SHA512',
            ecies_mac_digest: 'SHA512'
          }.freeze

          Contract Contracts::HashOf[Symbol, Maybe[String]] => Any
          # Initialize options for OpenSSL EC encryption
          #
          # @param clear_data (#clear_data)
          # @param public_key (#public_key)
          # @param ecies_options Hash with ECIES options
          #
          # @since 0.1.0
          def initialize(clear_data:, public_key:, ecies_options:)
            super(clear_data: clear_data)
            @public_key = public_key
            initialize_public_key!(public_key)
            initialize_ecies_options!(ecies_options || DEFAULT_ECIES_OPTIONS)
            initialize_encrypter!
          end

          Contract String => OpenSSL::PKey::EC
          # Initialize EC public key from public key string
          #
          # @param public_key (#public_key)
          #
          # @return [OpenSSL:PKey::EC] OpenSSL public key instance
          #
          # @since 0.1.0
          def initialize_public_key!(public_key)
            raise 'ERROR: No public key!' unless public_key

            @public_key = OpenSSL::PKey.read(public_key)
          end

          Contract Contracts::HashOf[Symbol, Or[String, Symbol]] => Any
          # Initialize ECIES options
          #
          # @param ecies_options Hash with ECIES options
          #
          # @since 0.1.0
          def initialize_ecies_options!(ecies_options)
            @ecies_cipher = ecies_options[:ecies_cipher]
            @ecies_digest = ecies_options[:ecies_digest]
            @ecies_mac_length = ecies_options[:ecies_mac_length]
            @ecies_kdf_digest = ecies_options[:ecies_kdf_digest]
            @ecies_mac_digest = ecies_options[:ecies_mac_digest]
          end

          Contract None => ECIES::Crypt
          # Initialize encrypter
          #
          # @return [ECIES::Crypt] encrypter instance
          #
          # @since 0.1.0
          def initialize_encrypter!
            @encrypter = ECIES::Crypt.new(
              cipher: ecies_cipher,
              digest: ecies_digest,
              mac_length: ecies_mac_length,
              kdf_digest: ecies_kdf_digest,
              mac_digest: ecies_mac_digest
            )
          end

          private

          Contract None => Contracts::ArrayOf[Contracts::HashOf[Symbol, String]]
          # Encrypting data
          #
          # @return [Array] encrypted data with empty hash in place of params
          #
          # @since 0.1.0
          def encrypt_data!
            [{ encrypted_data_string: encrypter.encrypt(public_key, clear_data) }, {}]
          end
        end
      end
    end
  end
end
