# frozen_string_literal: true

module Izokatu
  module Openssl
    # Namespace for OpenSSL public key classes
    module PublicKey
      # Namespace for OpenSSL public key EC classes
      module EC
        # OpenSSL EC keys generator
        class KeysGenerator < Izokatu::KeysGenerator
          # @return [String] cipher for keys generation
          attr_reader :cipher

          # Default options for OpenSSL EC keys generation
          DEFAULT_OPTIONS = {
            cipher: 'secp521r1'
          }.freeze

          Contract Contracts::HashOf[Symbol, String] => Any
          # Initialize option for OpenSSL EC keys generation
          #
          # @param cipher (#cipher)
          #
          # @since 0.1.0
          def initialize(cipher:)
            super()
            @cipher = cipher || DEFAULT_OPTIONS[:cipher]
          end

          Contract None => Contracts::HashOf[Or[*KEYS_SYMBOLS], String]
          # Performing generation of OpenSSL EC private and public keys
          #
          # @return [Hash] OpenSSL EC public and private keys
          #
          # @since 0.1.0
          def perform
            validate_ec_cipher!
            super.transform_values(&:to_pem)
          end

          private

          # Verifying EC cipher
          #
          # @raise RuntimeError
          #
          # @since 0.1.0
          def validate_ec_cipher!
            raise 'ERROR: Unknown EC cipher!' unless PBKEY_EC_CIPHERS.include?(cipher)
          end

          Contract None => OpenSSL::PKey::EC
          # Performing generation of OpenSSL EC private key
          #
          # @return [OpenSSL::PKey::EC] OpenSSL private key
          #
          # @since 0.1.0
          def generate_private_key
            OpenSSL::PKey::EC.new(cipher).generate_key
          end

          Contract OpenSSL::PKey::EC => OpenSSL::PKey::EC
          # Performing generation of OpenSSL EC private key
          #
          # @return [OpenSSL::PKey::EC] OpenSSL private key
          #
          # @since 0.1.0
          def generate_public_key(private_key)
            private_key_copy = OpenSSL::PKey::EC.new(private_key.public_key.group)
            private_key_copy.public_key = private_key.public_key
            private_key_copy
          end
        end
      end
    end
  end
end
