# frozen_string_literal: true

module Izokatu
  module Openssl
    module PublicKey
      module RSA
        # OpenSSL public key RSA decrypter
        class Decrypter < Izokatu::Decrypter
          # @return [String] private key string for decryption
          attr_reader :private_key

          Contract Contracts::HashOf[Symbol, String] => Any
          # Initialize options for OpenSSL RSA decryption
          #
          # @param encrypted_data (#encrypted_data)
          # @param private_key (#private_key)
          #
          # @since 0.1.0
          def initialize(encrypted_data:, private_key:)
            super(encrypted_data: encrypted_data)
            initialize_private_key!(private_key)
          end

          private

          Contract String => OpenSSL::PKey::RSA
          # Initialize RSA private key from private key string
          #
          # @param private_key (#private_key)
          #
          # @return [OpenSSL:PKey::RSA] OpenSSL private key instance
          #
          # @since 0.1.0
          def initialize_private_key!(private_key)
            raise 'ERROR: No private key!' unless private_key

            @private_key = OpenSSL::PKey.read(private_key)
          end

          Contract None => Contracts::HashOf[Symbol, String]
          # Decrypting data
          #
          # @return [Hash] decrypted data
          #
          # @since 0.1.0
          def decrypt_data!
            { decrypted_data_string: private_key.private_decrypt(encrypted_data) }
          end
        end
      end
    end
  end
end
