# frozen_string_literal: true

module Izokatu
  module Openssl
    module PublicKey
      module RSA
        # OpenSSL public key RSA encrypter
        class Encrypter < Izokatu::Encrypter
          # @return [String] public key string for decryption
          attr_reader :public_key

          Contract Contracts::HashOf[Symbol, String] => Any
          # Initialize options for OpenSSL RSA encryption
          #
          # @param clear_data (#clear_data)
          # @param public_key (#public_key)
          #
          # @since 0.1.0
          def initialize(clear_data:, public_key:)
            super(clear_data: clear_data)
            initialize_public_key!(public_key)
          end

          Contract String => OpenSSL::PKey::RSA
          # Initialize RSA public key from public key string
          #
          # @param public_key (#public_key)
          #
          # @return [OpenSSL:PKey::RSA] OpenSSL public key instance
          #
          # @since 0.1.0
          def initialize_public_key!(public_key)
            raise 'ERROR: No public key!' unless public_key

            @public_key = OpenSSL::PKey::RSA.new(public_key)
          end

          private

          Contract None => Contracts::ArrayOf[Contracts::HashOf[Symbol, String]]
          # Encrypting data
          #
          # @return [Array] encrypted data with empty hash in place of params
          #
          # @since 0.1.0
          def encrypt_data!
            raise 'ERROR: No public key!' unless public_key

            [{ encrypted_data_string: public_key.public_encrypt(clear_data) }, {}]
          end
        end
      end
    end
  end
end
