# frozen_string_literal: true

module Izokatu
  module Openssl
    module PublicKey
      # Namespace for OpenSSL public key RSA classes
      module RSA
        # OpenSSL RSA keys generator
        class KeysGenerator < Izokatu::KeysGenerator
          # @return [Integer] bit number for OpenSSL RSA encryption/decryption
          attr_reader :bit_number

          # Default options for OpenSSL RSA keys generation
          DEFAULT_OPTIONS = {
            bit_number: 4096
          }.freeze

          Contract Contracts::HashOf[Symbol, Pos] => Any
          # Initialize option for OpenSSL RSA keys generation
          #
          # @param bit_number (#bit_number)
          #
          # @since 0.1.0
          def initialize(bit_number:)
            super()
            @bit_number = bit_number || DEFAULT_OPTIONS[:bit_number]
          end

          Contract None => Contracts::HashOf[Or[*KEYS_SYMBOLS], String]
          # Performing generation of OpenSSL RSA private and public keys
          #
          # @return [Hash] OpenSSL RSA public and private keys
          #
          # @since 0.1.0
          def perform
            super.transform_values(&:to_pem)
          end

          private

          Contract None => OpenSSL::PKey::RSA
          # Performing generation of OpenSSL RSA private key
          #
          # @return [OpenSSL::PKey::RSA] OpenSSL private key
          #
          # @since 0.1.0
          def generate_private_key
            OpenSSL::PKey::RSA.new(bit_number)
          end

          Contract OpenSSL::PKey::RSA => OpenSSL::PKey::RSA
          # Performing generation of OpenSSL RSA private key
          #
          # @return [OpenSSL::PKey::RSA] OpenSSL private key
          #
          # @since 0.1.0
          def generate_public_key(private_key)
            private_key.public_key
          end
        end
      end
    end
  end
end
