# frozen_string_literal: true

module Izokatu
  module Rbnacl
    module PublicKey
      # RbNaCl public key encrypter
      class Encrypter < Izokatu::Rbnacl::Encrypter
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey] public key
        attr_reader :public_key
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey] private key
        attr_reader :private_key

        # RbNaCl public and private key classes
        RBNACL_KEY_CLASSES = [
          RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey,
          RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey
        ].freeze

        Contract Contracts::HashOf[Symbol, Or[String, Or[*RBNACL_KEY_CLASSES]]] => Any
        # Initializing option for encryption
        #
        # @param clear_data (#clear_data)
        # @param public_key (#public_key)
        # @param private_key (#private_key)
        #
        # @since 0.1.0
        def initialize(public_key:, private_key:, clear_data:)
          @public_key = public_key
          @private_key = private_key
          super(clear_data: clear_data)
        end

        Contract None => RbNaCl::Boxes::Curve25519XSalsa20Poly1305
        # Initializing option for encryption
        #
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305] encrypter instance
        #
        # @since 0.1.0
        def create_encrypter!
          raise 'ERROR: No public key!' unless public_key
          raise 'ERROR: No private key!' unless private_key

          @encrypter = RbNaCl::Box.new(public_key, private_key)
        end

        Contract None => Contracts::ArrayOf[Contracts::HashOf[Symbol, String]]
        # Encrypting data
        #
        # @return [Array] encrypted data with decrypter params
        #
        # @since 0.1.0
        def encrypt_data!
          [
            { encrypted_data_string: encrypter.encrypt(nonce, clear_data) },
            { nonce: nonce }
          ]
        end
      end
    end
  end
end
